/*

  This is a part of LSDynamic.dll source code.
  
  Copyright (C) 2001-2002 Sylvain Rouquette
  rouque_s@epita.fr
  http://www.epita.fr:8000/~rouque_s

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __DYN_H__
#define __DYN_H__

#include <list>
#include <vector>
#include <string>
#include "../lsapi/common.h"
#include "../lsapi/lswinbase.h"
#include "../lsapi/safestr.h"

using namespace std;

#define M_PI	3.1415926535f



// Alpha blending

#ifndef WS_EX_LAYERED
#define WS_EX_LAYERED           0x00080000
#define LWA_COLORKEY            0x00000001
#define LWA_ALPHA               0x00000002
#define ULW_ALPHA               0x00000002
#endif // ndef WS_EX_LAYERED

#define WM_ALPHA		(WM_USER + 0x42)

typedef BOOL (WINAPI *lpfnSetLayeredWindowAttributes)(HWND hWnd, COLORREF crKey, BYTE bAlpha, DWORD dwFlags);

enum e_dyn {dyn_none, dyn_linear, dyn_circular, dyn_move, dyn_command, dyn_pause, dyn_resume, dyn_win_stats};
enum e_relay {relay_default, relay_sync, relay_speedup, relay_giveup};


class CDyn;
class CCaption;
class CDynType;
class CDynMacro;
class CDynStorage;


typedef vector<string>				StringVector;
typedef list<CCaption *>			CaptionList;
typedef list<CCaption *>::iterator	CaptionListIterator;
typedef vector<CDyn *>				CDynVector;
typedef vector<CDynType *>			CDynTypeVector;
typedef vector<CDynMacro *>			CDynMacroVector;
typedef list<CDynMacro *>			CDynMacroList;
typedef list<CDynMacro *>::iterator	CDynMacroListIterator;


typedef struct s_pos
{
	int			x, y;
	bool		x_relative, y_relative;
	bool		enable;
} t_pos;

typedef struct s_delta
{
	float		x, y;
} t_delta;

typedef struct s_range_xy {
	t_pos		current;
	t_pos		initial;
	t_pos		final;
	t_delta		delta;
	bool		enable;
} t_range_xy;

typedef struct s_range_mono {
	int			current;
	int			initial;
	int			final;
	float		delta;
	bool		enable;
} t_range_mono;

class CDynType
{
public:
	CDynType();
	CDynType(const class CDynType &dyn);
	CDynType& operator=(const CDynType& dyn);
	~CDynType();
	void getCaption(const char *arguments);
	void copyCaption(CaptionList &captions);
	void getHWND(const char *token);

	e_dyn			type;
	int				is_running;
	bool			has_hwnd, has_caption;
	long			start_time, end_time;
	long			cur_time;
	string			name, hwnd;
	CaptionList		caption;
};


class CCaption
{
public:
	CCaption();
	CCaption(const char *n);
	CCaption(const class CCaption &cap);
	CCaption& operator=(const CCaption &cap);
	virtual ~CCaption();
	void update(const CDyn *dyn, bool update_pos);
	bool init(const CDyn *dyn);
	void restore();
	void restoreLayer();
	int  initAngle(t_pos &p);
	void getPos(t_range_xy &range, float d_delta);
	void setPos();
	virtual void updateAlpha();

	e_dyn		move;
	string		name, hwnd;
	HWND		hWnd;
	HDC			hdc;
	bool		internalAlpha, cornerRelative;

	t_range_xy	size;
	t_range_xy	pos;
	t_range_xy	center;

	t_range_mono ray;
	t_range_mono angle;
	t_range_mono alpha;
};

class CDyn
{
public:
	CDyn(e_dyn e, FILE *f, char *n);
	CDyn(const class CDyn &dyn);
	CDyn& operator=(const CDyn& dyn);
	~CDyn();
	void getPos(t_pos *pos, char *token1, char *token2, bool relative);
	void load(FILE *f, char *line);
	void init(CDynMacro *owner, CDynType *window);
	void update(long t, bool update_pos, bool reverse = false);

	CDynType	info;

	t_pos		size;
	t_pos		pos;
	t_pos		dest;
	t_pos		center;
	t_pos		c_dest;						// center dest
	int			r_dest;						// ray dest
	int			angle;
	bool		cornerRelative;

	t_range_mono alpha;

	int			acc, r_acc, c_acc, a_acc;	// accelerations

	float		delta_time, cur_delta_time;
	float		linear_acc, ray_acc, center_acc, angle_acc;
};


class CDynMacro
{
public:
	CDynMacro(FILE *f, char *n);
	CDynMacro(const class CDynMacro &dyn);
	CDynMacro& operator=(const CDynMacro& dyn);
	~CDynMacro();
	static e_dyn retDynType(const char *str);
	void load(FILE *f, char *line);
	void init(const char *arguments, int t);
	bool update();
	bool updateMovement(int t, bool update_pos);
	void restoreCaption();
	void relayDyn();

	bool			b_loop, b_one_exec;
	bool			b_restore, b_relay, is_paused, b_reverse;
	CDynType		info, args;
	CDynVector		dynVector;		// for dynMacroStack
	CDynTypeVector	dynTypeVector;	// info to compute dynVector
	StringVector	restore;
	StringVector	relay;
	int				timer;
	e_relay			relay_mode;
	int				is_relayed;		// 1: jump to end; 2: jump to time
	int				relay_time;
};


class CDynStorage : public Window
{
public:
	CDynStorage(HWND parentWnd, int& code);
	~CDynStorage();

	CDynVector		dynVector;
	CDynMacroVector	dynMacroVector;
	CDynMacroList	dynMacroStack;
	CaptionList		handleWindows;
	CaptionList		starCaption;
	StringVector	internalAlpha;
	bool			is_running, b_bang, b_timer;
	bool			storeBadHandle, has_alpha;
	bool			storeHandle, restoreLayer;
	bool			relativeWindowCenter;
	bool			b_layeredShortcut;
	lpfnSetLayeredWindowAttributes SetLayeredWindowAttributes;

	void createDyns(char *filename = NULL);
	void addDynMacro(FILE *f, char *n);
	void addDyn(FILE *f, e_dyn e, char *n);
	void destroyDyns();
	bool hasInternalAlpha(string &str);
	CDynMacro *dynBangCommand(HWND caller, const char *bangCommandName, const char *arguments);
	void dynRunning(HWND caller, const char *bangCommandName, const char *arguments);
	void dynChangeMoveType(HWND caller, const char *bangCommandName, const char *arguments);
	void dynPauseAll(HWND caller, const char *bangCommandName, const char *arguments);
	void dynResumeAll(HWND caller, const char *bangCommandName, const char *arguments);
	static BOOL CALLBACK enumWindowsProc(HWND hItWnd, LPARAM lParam);
	static BOOL CALLBACK enumWindowsProcStar(HWND hItWnd, LPARAM lParam);
	void getWindowStar(CDynType *dyn_type);
	void getWindow(CCaption *cap);
	void relay(const char *name);

	void exec(const char *str);
	void pause(const string &s);
	void resume(const string &s);

	virtual void windowProc(Message& message);
	void onCreate(Message& message);
	void onDestroy(Message& message);
	void onTimer(Message& message);
	void onGetRevId(Message& message);
	void onRefresh(Message& message);
};

extern CDynStorage* DynStorage;


extern int TimerUpdateDelay;


char *trimQuotes(char *buf);
bool getArgs(StringVector &sv, const char *arguments);

void LsboxModuleHook(HWND caller, const char *bangCommandName, const char *arguments);
void DynBangCommand(HWND caller, const char *bangCommandName, const char *arguments);
void DynRunning(HWND caller, const char *bangCommandName, const char *arguments);
void DynChangeMoveType(HWND caller, const char *bangCommandName, const char *arguments);
void DynPauseAll(HWND caller, const char *bangCommandName, const char *arguments);
void DynResumeAll(HWND caller, const char *bangCommandName, const char *arguments);

extern "C" {
	__declspec( dllexport ) int initModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType* wd);
	__declspec( dllexport ) int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath);
	__declspec( dllexport ) void quitModule(HINSTANCE dllInst);
}


#endif /* __DYN_H__ */
